#include <stdio.h>
#include <string.h>

#define UINT16(a)       (unsigned short)(a[0] | (a[1] << 8))
#define UINT32(a)       (unsigned long)(a[0] | (a[1] << 8) | (a[2] << 16) | (a[3] << 24))
#define SINT16(a)       (signed short)(a[0] | (a[1] << 8))
#define SINT32(a)       (signed long)(a[0] | (a[1] << 8) | (a[2] << 16) | (a[3] << 24))
#define MEMCMP(a,b)     memcmp(a, b, sizeof(a))

int main(int argc, char *argv[]) {
    struct {
        unsigned char id[4];
        unsigned char size[4];
        unsigned char type[4];
    } riff;
    struct {
        unsigned char id[4];
        unsigned char size[4];
        unsigned char format[2];
        unsigned char channels[2];
        unsigned char sampleRate[4];
        unsigned char bps[4];
        unsigned char blockAlign[2];
        unsigned char bitsWidth[2];
    } fmtChunk;
    struct {
        unsigned char id[4];
        unsigned char size[4];
        unsigned char data[0];
    } dataChunk;
    FILE *fp;

    if (argc > 1) {
        if ((fp = fopen(argv[1], "r")) != NULL) {
            fread(&riff, sizeof(riff), 1, fp);
            printf("ID=%.4s size=%lu Type=%.4s\n", riff.id, UINT32(riff.size), riff.type);
            if (MEMCMP(riff.id, "RIFF")) {
                printf("Not a riff file\n");
                return(1);
            }
            if (MEMCMP(riff.type, "WAVE")) {
                printf("Not a wave file\n");
                return(1);
            }
            fread(&fmtChunk, sizeof(fmtChunk), 1, fp);
            printf("ID=%.4s size=%lu format=0x%04x channels=%u sample_rate=%lu byte/sec=%lu block_align=%u bits_width=%u\n",
                fmtChunk.id, UINT32(fmtChunk.size), UINT16(fmtChunk.format), UINT16(fmtChunk.channels),
                UINT32(fmtChunk.sampleRate), UINT32(fmtChunk.bps), UINT16(fmtChunk.blockAlign),
                UINT16(fmtChunk.bitsWidth));
            fread(&dataChunk, sizeof(dataChunk), 1, fp);
            printf("ID=%.4s size=%lu data=\n", dataChunk.id, UINT32(dataChunk.size));
            {
                int ch;
                int by;
                union {
                    unsigned char s1[16];
                    unsigned char s2[8][2];
                    unsigned char s4[4][4];
                } buff; 
                int nu;

                by = UINT16(fmtChunk.bitsWidth) / 8;
                ch = UINT16(fmtChunk.channels);
                nu = 0;
                while(fread(&buff, by, ch, fp)) {
                    int i;

                    nu++;
                    printf("%d,", nu);
                    for(i = 0; i < ch; i++) {
                        switch(by) {
                        case 1:
                            printf("%d ", buff.s1[i]);
                            break;
                        case 2:
                            printf("%d ", SINT16(buff.s2[i]));
                            break;
                        default:
                            printf("%ld ", SINT32(buff.s4[i]));
                            break;
                        }
                    }
                    printf("\n");
                }
            }
            fclose(fp);
        }
    } else {
        printf("usage: %s <wav file>\n", argv[0]);
    }
    return(0);
}

