#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#define FS              44100	// Sampling frequency

#define UINT16(a)       (unsigned short)(a[0] | (a[1] << 8))
#define UINT32(a)       (unsigned long)(a[0] | (a[1] << 8) | (a[2] << 16) | (a[3] << 24))
#define SINT16(a)       (signed short)(a[0] | (a[1] << 8))
#define SINT32(a)       (signed long)(a[0] | (a[1] << 8) | (a[2] << 16) | (a[3] << 24))
#define MEMCMP(a,b)     memcmp(a, b, sizeof(a))
#define MEMCPY(a,b)     memcpy(a, b, sizeof(a))
#define SET_UINT16(a,b) a[0]=(unsigned char)(b), a[1]=(unsigned char)((b)>>8)
#define SET_UINT32(a,b) a[0]=(unsigned char)(b), a[1]=(unsigned char)((b)>>8), a[2]=(unsigned char)((b)>>16), a[3]=(unsigned char)((b)>>24)
#define SET_SINT16(a,b) a[0]=(unsigned char)(b), a[1]=(unsigned char)((b)>>8)
#define SET_SINT32(a,b) a[0]=(unsigned char)(b), a[1]=(unsigned char)((b)>>8), a[2]=(unsigned char)((b)>>16), a[3]=(unsigned char)((b)>>24)


int main(int argc, char *argv[]) {
    struct {
        unsigned char id[4];
        unsigned char size[4];
        unsigned char type[4];
    } riff;
    struct {
        unsigned char id[4];
        unsigned char size[4];
        unsigned char format[2];
        unsigned char channels[2];
        unsigned char sampleRate[4];
        unsigned char bps[4];
        unsigned char blockAlign[2];
        unsigned char bitsWidth[2];
    } fmtChunk;
    struct {
        unsigned char id[4];
        unsigned char size[4];
        unsigned char data[0];
    } dataChunk;
    FILE *fp;
    double f;	// Output frequency
    int    s;	// Sec
    int    n;	// Total sample #

    if (argc > 2) {
        f = atoi(argv[2]);
    } else {
        f = 440;
    }
    if (argc > 3) {
        s = atoi(argv[3]);
    } else {
        s = 10;
    }
    n = FS * s;

    MEMCPY(riff.id, "RIFF");
    SET_UINT32(riff.size, n * 2 + 36);
    MEMCPY(riff.type, "WAVE");
    MEMCPY(fmtChunk.id, "fmt ");
    SET_UINT32(fmtChunk.size, 16);
    SET_UINT16(fmtChunk.format,     0x0001);
    SET_UINT16(fmtChunk.channels,   1);
    SET_UINT32(fmtChunk.sampleRate, FS);
    SET_UINT32(fmtChunk.bps,        FS * 2);
    SET_UINT16(fmtChunk.blockAlign, 2);
    SET_UINT16(fmtChunk.bitsWidth,  16);
    MEMCPY(dataChunk.id, "data");
    SET_UINT32(dataChunk.size, n * 2);

    if (argc > 1) {
        if ((fp = fopen(argv[1], "w")) != NULL) {
            int i;

            fwrite(&riff, sizeof(riff), 1, fp);
            fwrite(&fmtChunk, sizeof(fmtChunk), 1, fp);
            fwrite(&dataChunk, sizeof(dataChunk), 1, fp);
            for(i = 0; i < n; i++) {
                unsigned char data[2];
                short a;

                a = sin((double)i / (double)FS * f * 2.0 * M_PI) * 32767.0;
                SET_SINT16(data, a);
                fwrite(data, sizeof(data), 1, fp);
            }
            fclose(fp);
        }
    } else {
        printf("usage: %s <wav file> [<frequency> [sec]]\n", argv[0]);
    }
    return(0);
}

